function Run_SNPLS(Input)
%
% This algorithm is only used for two input blocks, X, Y with the same
% rows, and users should have one networks G referring to the interactions
% between the features of X.
% 
% example: 
% >> clear all
% >> Run_SNPLS(Input)
%
% INPUT:
%
% Input : a structure variable, including five variables, Input.data,
%         Input.XBlockInd, Input.YBlockInd, Input.netAdj, Input.NCluster.
%
% Input.data : it should be a (M x N) matrix. Each row represents a sample,
%              and each column is a feature. Here, user should combine all 
%              the data to be integrated together by columns.          
% Input.XBlockInd : it should be a (1 x 2) matrix. Each row represents the
%                  column index(start index, end index) of X block in the 
%                  Input.data.
% Input.YBlockInd : it should be a (1 x 2) matrix. Each row represents the
%                  column index(start index, end index) of Y block in the 
%                  Input.data.
% For example, 
% there are two types of data X,Y which share the same rows, that
% is X(M x N1),Y(M x N2), thus Input.data = [X1,X2], where
% N = N1+N2. And Input.XBlockInd = [1 N1]; Input.YBlockInd = [(N1+1),
% (N1+N2)].
%
% Input.netAdj : it is the adjacent matrix for the network which represents
%                the relationship between all the features in the input data.
%                For 'SNPLS' algorithm, user should provide one network
%                G for the interactions between the features in X. And,
%                A(N1 x N1) is the adjacent matrices for G. Thus,
%                Input.netAdj = [A 0(N1 x N2); 0(N2 x N1) 0(N2 x N2)],
%                where 0(N1 x N1) is a zeros matrix with N1 rows and N1
%                columns.
% Input.NCluster : it is the number of modules user wants to identify.
%

if nargin == 0
    help Run_SNPLS
    return
end

%%%%%%%%%%%%%%%%%%%%%% Load data X,Y for input %%%%%%%%%%%%%%%%%%%%
dMat = Input.data;
XInd = Input.XBlockInd;
YInd = Input.YBlockInd;
nbX = size(XInd,1); % nbX : #blocks in X
nbY = size(YInd,1); % nbY : #blocks in Y
if(nbX ~= 1 || nbY ~= 1)
    error('This algorithm is only used for one input block X and one response block Y!')
end
X = dMat(:,XInd(1,1):XInd(1,2));
Y = dMat(:,YInd(1,1):YInd(1,2));
clear XInd YInd

[rNet,cNet] = size(Input.netAdj);
[rdMat,cdMat] = size(dMat);
if(cdMat ~= cNet || rNet ~= cdMat)
    error('The dimension of adjacent network does not match with the input matrix!')
end
clear cdMat rdMat rNet cNet dMat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ResultsFile = 'SNPLS_Results';
if ~isdir(ResultsFile)
    mkdir(ResultsFile);
end

% plot the heatmap of original data X and Y.
colormap_type = 'blue-white-red';
SNPLS_plot_XY(X,Y,100,'Original data',colormap_type);

saveas(gcf,[ResultsFile '/Original_data.fig'])
close(figure(100))

%%%%%%%%% You may define your own combinations of parameters here %%%%%%%%%
% For each parameter, please use semicolon to separate different values for
% this parameter.
thrNet_list = [1;10]; % Thresholds for the constraint about network.
thrXc_list = [0.01;0.03;0.05]; % Thresholds for the sparsity of variable g.
thrYc_list = [1;3;5]; % Thresholds for the sparsity of variable d.
delta_list = [1;10]; % For the equality constraint.
%
module_ThrX = [3,0.005]; 
module_ThrY = [3,0.02];
module_ThrS = [2,0.05];
thrd_module = [module_ThrS;module_ThrX;module_ThrY];
% Thresholds for identifying comodule, you may define your own 
% thresholds according to your input data or the size of comodules
% you want to identified. module_ThrX is a 1 x 2 vector, the first 
% element > 0 and the secend one is in [0,1]. 
%
% For example, module_ThrX = [3,0.005], 
% that means your identified module for the varibles of X will be 
% the components with zscore > 3; if the identified component is empty,
% then we use the second standard to dentify module, that is, 
% the components whose absolute values are the top (0.005 x size(X,2)).
% The details about the procedure refer to function SNPLS_comodule.m
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
params = SNPLS_params(thrNet_list,thrXc_list,thrYc_list,delta_list);
params.nfactor = Input.NCluster; % #comodules you prefer to identify.
params.module = thrd_module;
params.nfold = 5; % Used for cross-validation.
% params: Structure variable, including all necessary parameters needed to 
% run this program, that is,
%
% params.combparams: A matrix with all the possible combinations of these 
%                    (nbX+nbY+1) parameters.
% params.maxIter: The maximal iteration step.
% params.epsilon: The minimal number to judge the stop of iteration.
% params.param: Cell variable, using Cross-Validation to select proper
%               parameters from them.
% params.nfold: The number of folds used for cross-validation.
% params.nfactor: #comodules you prefer to identify.
% params.module: Thresholds for identifying comodule. 

%%%%%%%%%%%% Run SNPLS algorithm, and output the comodules %%%%%%%%%%%%
netAdj = Input.netAdj;
clear Input
[nfactor,G,D,U,V,XX,YY,Comodule,params,ObjVal,MSE] = SNPLS_comodule(X,Y,netAdj,params);


save([ResultsFile '/' ResultsFile '.mat'],'nfactor','G','D','U','V','XX','YY',...
    'ObjVal','MSE','Comodule','params');

% Output the co-modules to a text file
OutputModule2TXT(Comodule(:,1:3),[ResultsFile '/' ResultsFile]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Module size distribution
nSamples = size(X,1);
Comodule = Comodule(:,1:3);
figure_title = 'Module size distributions';
Ind = [1,size(X,2);(1+size(X,2)),(size(X,2)+size(Y,2))];
SNPLS_plot_distribution(nSamples,Ind,Comodule,400,figure_title);

figure_title = 'Module_size_distributions';
saveas(gcf,[ResultsFile '/' figure_title '.fig'])
close(figure(400))

end

