function [success,g,d,u,v,XX,YY,objVal] = SNPLS_algorithm(X,Y,L,param)
% This is the SNPLS algorithm for the integration of one input block X and
% one response block Y.
%
% INPUT :
%
% X: (objects x all X-variables) single X-data-block
% Y: (objects x all Y-variables) single Y blocks with the same rows
%     of X.
% L: (all X-variables x all X-variables) the normalized Laplacian matrix
%     for the adjacent matrix of X-variables.
% param: Structure variable, including all necessary parameters needed to 
%        run this program, that is,
% param.thrNet: The parameter for the network constraint. 
% param.thrXc: The parameter for the sparsity of g. 
% param.thrYc: The parameter for the sparsity of d. 
% param.delta: The parameter for the equality constraint. 
% param.maxIter: The maximal iteration steps. 
% param.epsilon: Precision for convergence.   
%
% OUTPUT :
%
% success: a binary variable, to judge for calculating successfully or not.
% g : (X-variables x 1) X-block weights
% d : (Y-variables x 1) Y-block weights
% u,v : (objects x 1) score variables for X,Y
% XX : new matrices (objects x X-variables)
% YY : new matrix (objects x Y-variables)
% objVal : a vector, recording the value of objective function in each
%          iteration step
%
[rX,cX] = size(X);
[rY,cY] = size(Y);
[rL,cL] = size(L);

if((rL ~= cL) || (rL ~= cX))
    error('Please test the dimension of Laplacian Matrix(square matrix).')
end

if(rX ~= rY)
    error('Please input matrices X and Y with same number of rows.')
end

%%%%%%%%%%%%%%%%%%%%%%% Preprocess the data %%%%%%%%%%%%%%%%%%%%%%%%%
X = zscore(X);
Y = zscore(Y);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Precision for convergence
tol = param.epsilon;
maxiter = param.maxIter;
% Thresholding: lambda1,2,3;
thrd_net = param.thrNet; % for network
thrd_g = param.thrXc; % for g
thrd_d = param.thrYc; % for d
delta = param.delta; % for equality constraint
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
new_L = -L + diag(ones(cX,1));
%%%%%%%%%%%%%%%%%%%%%%%%%% Initialization %%%%%%%%%%%%%%%%%%%%%%%%%%%
XX = zeros(rY,cX);
YY = zeros(rY,cY);
success = 0;
iter = 0;
%%%%%%%%%%%%%%%%%% Main body of algorithm %%%%%%%%%%%%%%%%%%%%%%
% step 1: initial the weight vector g and d;
[success_PLS,g,d,u,v] = PLS(X,Y);
if(~success_PLS)
    disp('Standard PLS algorithm failed!')
    % g = ones(cX,1)/sqrt(cX);
    % u = X*g;
end

% update each variable iteratively.
res = zeros(maxiter,1);
objVal = zeros(maxiter,1);
resi = Inf;

while (resi > tol) && (iter < maxiter)
    iter = iter + 1;
    u_old = u;
    % step 2: update the d;
    temp = Y'*u/rY;
    d = thresholding(temp,thrd_d,'d');
    d = d/norm(d);
    err = NaNerrorCheck(d,'d');
    if (err),
        return;
    end
    % return: directly skip the below code,return the
    % function SNPLS,that is, success = 0;
    v = Y*d;
    % step 3: fixed d, it is a problem about g, then we obtain g = ?
    temp = (X'*v)/rX;
    %%%%
    for k = 1:cX
        z = temp(k)+2*thrd_net*new_L(k,:)*g;
        g(k) = thresholding(z,thrd_g,'g')/(2*(thrd_net + delta));
    end
    %%%%
    g = g/norm(g);
    err = NaNerrorCheck(g,'g');
    if (err),
        return;
    end
    u = X*g;
    res(iter) = sum((u_old - u).^2);
    resi = res(iter);
    obj = (u'*v)/rY - thrd_g*sum(abs(g)) - thrd_d*sum(abs(d)) - ...
        thrd_net*g'*L*g;
    objVal(iter) = obj;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if iter == maxiter
    s = ['SNPLS Algorithm: Maximum number of iterations (' num2str(maxiter) ...
        ') reached before convergence'];
    fprintf(1,'%s\n',s);
end

p = X'*u/(u'*u);
q = Y'*v/(v'*v);
XX = X - u*p';
YY = Y - v*q';
clear X Y
success = 1;
end


