function [nfactor,G,D,U,V,XX,YY,Comodule,params,ObjVal,MSE] = ...
    SNPLS_comodule(X,Y,netAdj,params)
% The main body of SNPLS framework.
%
% INPUT:
%
% X: matrix (objects x X-variables)
% Y: matrix (objects x Y-variables)
% netAdj: ((X-variables+Y-variables) x (X-variables+Y-variables)) The
%         adjacency matrix for network representing the relations between 
%         the variables of X and Y.        
% params: Structure variable, including all necessary parameters needed to 
% run this program, that is,
%
% params.combparams: A matrix with all the possible combinations of these 
%                    (nbX+nbY+1) parameters.
% params.maxIter: The maximal iteration step.
% params.epsilon: The minimal number to judge the stop of iteration.
% params.param: Cell variable, using Cross-Validation to select proper
%               parameters from them.
% params.nfold: The number of folds used for cross-validation.
% params.nfactor: #comodules you prefer to identify.
% params.module: Thresholds for identifying comodule. 
%
% OUTPUT:
%
% nfactor: the number of successfully identified comodules 
% G: (X-variables x nfactor) each column refers to the X-block 
%    weigths --- g.
% D: (Y-variables x nfactor) each column refers to Y-block weights --- d.
% U: (objects x nfactor) each column refers to X-block scores --- u.
% V: (objects x nfactor) each column refers to Y-block scores --- v.
% XX,YY: the new matrices after substracting.
% Comodule: cell(nfactor x 3),identified by the algorithm
% params: different from the input parameters-params, added other
%         information.
%
if nargin == 0
    help SNPLS_comodule
    return
end

ResultsFile = 'SNPLS_Results';
if ~isdir(ResultsFile)
    mkdir(ResultsFile);
end

%%%%%%%%%%%%%%%%% Normalized Laplacian Matrix for netAdj %%%%%%%%%%%%%%%%%%
[rX,cX] = size(X);
objAdj = netAdj(1:cX,1:cX);
diagEle = diag(objAdj);
objAdj = objAdj - diag(diagEle);
D = sum(objAdj,1);
pA = triu(objAdj,1);
[r,c,tempVal] = find(pA);
L = sparse(r,c,-1./sqrt(D(r).*D(c)),cX,cX);
L = L + L' + sparse(1:cX,1:cX,1,cX,cX);
clear tempVal r c netAdj D pA objAdj diagEle
%%%%%%%%%%%%%%%%%%%%%%%%%%% Test %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[rY,cY] = size(Y);

if(rX ~= rY)
    error('Please input matrices X and Y with same number of rows.')
end

%%%%%%%%%%%%%%%%%%%%%% Initialization %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nLV = params.nfactor;
nparameter = length(params.param);
G = zeros(cX,nLV);
D = zeros(cY,nLV);
U = zeros(rX,nLV);
V = zeros(rX,nLV);

ObjVal = zeros(params.maxIter,nLV);
MSE = zeros(nLV,1);

Comodule = cell(nLV,6);
params.cv_scores = zeros(nparameter,nLV);
params.paramsidx_used = zeros(nLV,1);
nfactor = 0;

%%%%%%%%%%%%%%%%%%%%%%% Preprocess the data %%%%%%%%%%%%%%%%%%%%%%%%%
X = zscore(X);
Y = zscore(Y);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nfold = params.nfold; % number of folds for cross-validation.
params.randRowPartitions = rand_nFold(rX,nfold);
colormap_type = 'blue-white-red';

for a = 1:nLV
    fprintf('Latent Variable %d\n', a);
    %%%%%%%%%%%%%%%%%%%%% Select proper parameters %%%%%%%%%%%%%%%%%%%%%
    [param_idx,cv_scores_onefactor] = SNPLS_select_param(X,Y,L,params);
    %%%%%%%%%%%%%%%%%%%%%% Run the SNPLS algorithm %%%%%%%%%%%%%%%%%%%%
    [success,g,d,u,v,XX,YY,objVal] = SNPLS_algorithm(X,Y,L,params.param{param_idx});
    ObjVal(:,a) = objVal;
    if (success),
        nfactor = nfactor + 1;
        %%%%%%%%%%%% Plot the reordered X and Y %%%%%%%%%%%%%%
        vectorForRank.u = u;
        vectorForRank.v = v;
        vectorForRank.g = g;
        vectorForRank.d = d;
        SNPLS_plot_XY(X,Y,200 + a,...
            ['Reordered data by SNPLS solution vector ' num2str(nfactor)],...
            colormap_type,vectorForRank);
        saveas(gcf,[ResultsFile '/Reordered_data_' num2str(nfactor) '.fig'])
        close(figure(200+a))
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Identify comodules by means of the variables.
        moduleGThrd = params.module(2,:); % a vector : 1 x 2
        moduleDThrd = params.module(3,:);
        moduleSThrd = params.module(1,:);
        
        [gind,gthrdType] = identify_module(g,moduleGThrd);
        newg = newx(g,gind);
        
        [dind,dthrdType] = identify_module(d,moduleDThrd);
        newd = newx(d,dind);
        
        u = X*newg;
        v = Y*newd;
        su = u/sqrt(u'*u);
        sv = v/sqrt(v'*v);
        
        [sind,sthrdType] = identify_module(su + sv,moduleSThrd);

        newu = newx(u,sind);
        newv = newx(v,sind);
        
        p = X'*newu/(newu'*newu);
        q = Y'*newv/(newv'*newv);
        XX = X - newu*p';
        YY = Y - newv*q';
        
        MSE(a) = sum(sum(XX.^2))/(rX*cX) + sum(sum(YY.^2))/(rX*cY);
        
        Comodule{a,1} = sind;
        Comodule{a,2} = gind;
        Comodule{a,3} = dind;
        Comodule{a,4} = sthrdType;
        Comodule{a,5} = gthrdType;
        Comodule{a,6} = dthrdType;
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        vectorForRank.comodule = Comodule(a,1:3);
        SNPLS_plot_results(X,Y,300 + a,['Identified comodule ' num2str(nfactor)],...
            colormap_type,vectorForRank)
        
        saveas(gcf,[ResultsFile '/Identified_comodule_' num2str(nfactor) '.fig'])
        close(figure(300+a))
        
        clear vectorForRank
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Update matrices X1,X2,X3,Y for identifying the next comodule
        X = zscore(XX);
        Y = zscore(YY);
        % Record the values of related variables.
        G(:,a) = g;
        D(:,a) = d;
        U(:,a) = u;
        V(:,a) = v;
        % Record the selected parameters by CV method
        params.paramsidx_used(a,1) = param_idx;
        params.cv_scores(:,a) = cv_scores_onefactor;
        save([ResultsFile '/SNPLS_' num2str(nfactor) '.mat'],'g','d',...
           'u','v','newg','newv','newu','newd','param_idx',...
           'cv_scores_onefactor','X','Y','objVal','gind',...
           'dind','sind','Comodule')
    else
        return;
    end
    
end

XX = X;
YY = Y;

end

function randpartitions = rand_nFold(n,nfold)
% randomly partition samples into 'nfold' folds for cross-validation.
randpartitions = cell(nfold,1);
sizePartition = fix(n/nfold);
r = randperm(n);
base = 1;
for i = 1:(nfold-1),
    randpartitions{i,1} = r((base:(base+sizePartition-1)))';
    base = base + sizePartition;
end
randpartitions{nfold,1} = r(base:n)';
end

function [ind,thrdtype] = identify_module(x,THRD)
% select the index of features as our members of module.
percent = THRD(2);
thrd = THRD(1);

len = length(x);
zx = zscore(abs(x));
ind = find(zx > thrd);
thrdtype = 'adjusted';
if(isempty(ind))
    tempLen = ceil(len*percent);
    [szx,sind] = sort(zx,'descend');
    ind = sind(1:tempLen);
    thrdtype = 'top';
end
ind = sort(ind);
end

function sx = newx(x,ind)
sx = zeros(length(x),1);
sx(ind) = x(ind);
end


