function cv_score = SNPLS_getCVscore(X,Y,L,param,randRowPartitions)
% For the given param, using cross-validation to compute the cv score.
%
% INPUT:
%
% X: matrices (objects x X-variables)
% Y: matrix (objects x Y-variables)
% L: normalized Laplacian matrix for the network.
% param: a structure containing all necessary parameters needed to run
%         this program, only one group.
% randRowPartitions: cell(nfold x 1), samples partition.
%
% OUTPUT:
% cv_score: a number, the cross-validation score for the given parameter 
%           combination ('param')
%
if nargin == 0
   help SNPLS_getCVscore
   return
end

nfold = length(randRowPartitions);
cv_score = 0;
 
for k = 1:nfold,
    % for each fold, obtain the training and test groups.
    rowi_selected = randRowPartitions{k};
    rowi_rest = union_cells(randRowPartitions,setdiff(1:nfold,k));
    X_train = X(rowi_rest,:);
    X_test = X(rowi_selected,:);
    Y_train = Y(rowi_rest,:);
    Y_test = Y(rowi_selected,:);
    
    % on training data, compute the related variables.
    [success,g,d,u,v,XX,YY] = SNPLS_algorithm(X_train,Y_train,L,param);
    clear u v XX YY
    
    % on test data, compute the cv-score.
    if (success)
        u = X_test*g;
        v = Y_test*d;
        p = X_test'*u/(u'*u);
        q = Y_test'*v/(v'*v);
        dist_X = approx_dist(X_test, u, p);
        dist_Y = approx_dist(Y_test, v, q);
        cv_score = cv_score + dist_X + dist_Y;
        clear u v p q dist_X dist_Y
    else
        cv_score = Inf;
        return;
    end  
end
end

%%%%%%%%%%%%%  Functions Here %%%%%%%%%%%%%%%%%%%%%%%
% 
function d = approx_dist(X,u,v)
% Compute the difference between two matrices X, u'*v.
%
% INPUT:
% X: mxn matrix  
% u: mx1 vector
% v: nx1 vector
%
% OUTPUT:
% d: 1x1 vector
%
[m,n] = size(X);
d = sum(sum((X-u*v').^2));
d = d/(m*n);
end

function array = union_cells(cellarray,listi) 
% Array in each cell is a column vector.
n = length(cellarray);
if (max(listi) > n)
    error('union_cells error: max of listi > length of cell array.');
end
array = [];
for i = 1:length(listi),
    array = [array; cellarray{listi(i)}];
end
end


