function C = exp_colormap(colors, N)
% a simple function that returns a colormap, C, for visualizing gene
% expression. C is just a N x 3 matrix [R G B] describing the range 
% of color values.
%
% example usage:
% >> C = exp_colormap('blue-yellow',64);
% >> colormap(C);
% 
% INPUT:
%
% color: options for it are: 'blue-yellow'; 'green-red'; 'yellow'; 
%                            'blue-white-red'; 'default'.
% N: the number of degrees of color to use.  the default is 64.
%
% OUTPUT:
% C: it returns a [N x 3] RGB colormap.
%
% generally speaking, the two-colored maps are appropriate for visualizing
% expression data which is normalized to a reference condition, e.g. to show
% whether expression is lower (blue or green) or higher (yellow or red) than
% the reference.
%
% the single-color yellow map ('yellow') is appropriate for displaying
% levels of gene expression which are not compared (necessarily) to a single
% reference, and this is similar to the colormap used in the D-chip
% expression analysis software.
% 
% the colormaps returned range monotonically.
%  
  
if nargin < 2 
  N = 64;
end

if nargin < 1
  colors = 'green-red';
end

X = 0.5: -1/(N-1):-0.5;
X = abs(X).*2;

switch colors
case {'green-red'}
  R = [X(:, 1:N/2) zeros(1,N/2)];
  G = [zeros(1, N/2) X(:,(N/2 + 1):N)];
  B = zeros(1,N);

case {'blue-yellow'} 
  R = [zeros(1,N/2) X(:,(N/2 + 1):N)];
  B = [X(:,1:N/2) zeros(1,N/2)];
  G = [zeros(1,N/2) X(:,(N/2 + 1):N)];

case {'yellow'} 
  X = 0:1/(N - 1):1;
  R = X;
  B = zeros(1,N);
  G = X;

case {'blue-white-red'}
  R = [X(:,(N/2 + 1):N) ones(1,N/2) ];
  G = [X(:,(N/2 + 1):N) X(:, 1:N/2) ];
  B = [ones(1,N/2) X(:, 1:N/2) ];
  
case {'default'}
  C = colormap(summer);
  return;
  
otherwise
 error([colors ' is not a known option for coloring.']);
end

C = [R' G' B'];
end
